
################################################################################

#  Taller GBIF.ES: Manejo, visualizacin y anlisis de datos en ecologa con R
# Code by: Julia Chacn Labella
# Date: November 2018
# email: chacon.julia@inia.es

################################################################################
# TEMA 3.-  Tipos de objetos (continuacin)
################################################################################

# Introduccin a los tipos de datos y estructuras
# 1. vectores
# 2. listas
# 3. arrays
# 4. matrices
# 5. data frames

################################################################################
################################################################################
# 5) Marcos de datos  (data frames)
################################################################################

# son LISTAS de variables del mismo numero de FILAS

# para almacenar datos en R. Lo podemos traducir como TABLA, con una estructura
# bidimensional (filas x columnas), similar a una matriz, pero la diferencia es 
# que puede contener diferentes tipos de elementos (numeros, texto,...no todos 
# los elementos han de ser iguales como en una matriz)

# Una de las limitaciones del data frame es que todas las columnas tienen que 
# tener la misma longitud, si no...sera UNA LISTA... es decir
# una coleccion de objetos o variables de cualquier, tipo, longitud o estructura

# Creamos un data frame:
? data.frame #consultamos la informacin de ayuda

especies <- c("Iris_setosa", "Iris_virginica", "Iris_versicolor")
ID <- c(1:3)
presencia <- c(TRUE, TRUE, FALSE)

data.frame(especies, ID, presencia)

data.frame(especies = c("Iris_setosa", "Iris_virginica", "Iris_versicolor"),
           ID = c(1:3), presencia = c(TRUE, TRUE, FALSE))

midataframe <- data.frame(especies, ID, presencia)

class(midataframe) ##la funcion class nos indica que tipo de objeto es
mode(midataframe) 
is.data.frame(midataframe)   #funcion is.data.frame
is.list(midataframe)        # funcion is.list


# un data frame tambien es una lista. 
# Pero una lista no es un data.frame. 
# Para serlo: 
# el requerimiento es que todos los elementos tengan la misma longitud
midf <- midataframe
midf

#  COMO ACCEDER A UN DATA FRAME
# consulta de columnas 
midf[1]
midf[2]
midf[3]

# consultas de columnas que te devuelve un vector
midf[,2]
midf[,3]
midf[,1]

# consulta por filas que te devuelve un vector
midf[1,]
midf[2,]
midf[3,]

midf[,1]

dimnames(midf)
dim(midf)

# los nombres de un data frame es el nombre de las variables que contiene
# es decir, el nombre de las columnas

names(midf)
rownames(midf)
colnames(midf)

# con el simbolo $ accedo a las variables, ejemplo especies

midf$especies #son vectores con niveles (levels)
midf[,1]
midf$ID

levels(midf$especies)
levels(midf$presencia)# consulta de los levels (tipos de entradas)
table(midf$presencia) # me hace una tabla de valores que aparecen

class(midf$presencia) # clase de cada variable dentro del data.frame
class(midf$especies)
class(midf$ID)

#   podemos transformar una matriz en data frame??
m <- matrix(data=c(101:112), nrow=3, ncol=4) # crear una matriz
m
# SI
data.frame(m)
# o de otra manera...
as.data.frame(m)
# o cambiar un data.frame a matriz convirtiendo todos al mismo tipo, por ejemplo 
# caracter
as.matrix(midf)

# de inters
is.data.frame(midf)
!is.matrix(midf)
!is.matrix(m)
is.data.frame(m)

# Factor, Table, Rep
# factor (categorias)
color.flor <- c("rosa", "blanca", "blanca", "azul", "rosa")
color.flor
is.factor(color.flor)

color.flor <- factor(c("rosa", "blanca", "blanca", "azul", "rosa")) 
color.flor
is.factor(color.flor)
#directamente te asigna niveles (levels)

# La funcin REP
# Podis explorar la funcin rep() ms en profundidad en la help de R
color.flor <- rep(c("rosa", "blanca", "blanca", "azul", "rosa"), times=10)
color.flor

# un factor son los elementos de este vector ordenados.
# nos ayudan a condensar la informacin cuando tenemos muchas observaciones
# y cuando trabajamos con valores categoricos

color.flor<- factor(color.flor)
levels(color.flor)
table(color.flor) # me hace una tabla resumen
summary(color.flor)

########
midf$especies

midf$especies[3]
midf$ID
midf$ID[1]

midf[,1][1]
midf[1,]
midf[1,][2]<-150
midf
